"""AWS Login utilities"""
import os
import logging
import boto3
from botocore.config import Config
logger = logging.getLogger(__name__)


def get_boto3_client(service: str, profile: str = None, region="us-east-1") -> boto3.Session.client:
    """Get a boto3 client for a given service"""
    logging.getLogger('botocore').setLevel(logging.CRITICAL)
    session_data = {"region_name": region}
    if profile:
        session_data["profile_name"] = profile
    session = boto3.Session(**session_data)

    config = Config(connect_timeout=5, retries={"max_attempts": 10})
    if os.environ.get('LOCALSTACK_ENDPOINT_URL'):
        client = session.client(service, config=config, endpoint_url=os.environ.get('LOCALSTACK_ENDPOINT_URL'))
    else:
        client = session.client(service, config=config, endpoint_url=os.environ.get('LOCALSTACK_ENDPOINT_URL'))
    logger.debug(f"{client.meta.endpoint_url} in {client.meta.region_name}: boto3 client login successful")
    return client


def get_current_account_id(sts_client: boto3.Session.client) -> str:
    """Get the current account ID"""
    response = sts_client.get_caller_identity()
    current_account_id = response.get("Account")
    return current_account_id


def get_target_account_credentials(target_account_role_name: str, target_account_id: str,
                                   role_session_name: str = "HotDogsAreSandwiches", profile: str = None):
    """
    Get a boto3 client for a given AWS service

    :param profile:
    :param role_session_name: AssumeRole session name
    :param target_account_role_name: The name of the target account role
    :param target_account_id: The target account ID
    :return:
    """
    default_region = "us-east-1"
    session_data = {"region_name": default_region}
    if profile:
        session_data["profile_name"] = profile
    session = boto3.Session(**session_data)
    config = Config(connect_timeout=5, retries={"max_attempts": 10})
    sts_client = session.client('sts', config=config)

    acct_b = sts_client.assume_role(
        RoleArn=f"arn:aws:iam::{target_account_role_name}:role/{target_account_id}",
        RoleSessionName=role_session_name
    )

    aws_access_key_id = acct_b['Credentials']['AccessKeyId']
    aws_secret_access_key = acct_b['Credentials']['SecretAccessKey']
    aws_session_token = acct_b['Credentials']['SessionToken']

    return aws_access_key_id, aws_secret_access_key, aws_session_token



import os


def is_running_in_aws_lambda() -> bool:
    """Detects if running in AWS Lambda"""
    return os.environ.get("LAMBDA_TASK_ROOT") is not None and os.environ.get("AWS_EXECUTION_ENV") is not None


def is_running_in_sam_cli_local() -> bool:
    """Detects if running in AWS SAM locally"""
    return os.environ.get("AWS_SAM_LOCAL") is not None



import subprocess  # nosec - skip B404:blacklist
import logging
import shlex
import os
import warnings
from typing import Union
from click.testing import CliRunner, Result
logger = logging.getLogger(__name__)


def run_shell_command(
        command: str,
        shell: bool = True,
        wait_for_finish: bool = True,
        env_vars: dict = None,
        timeout: int = None
) -> Union[subprocess.Popen, subprocess.CompletedProcess]:
    """
    wrapper for subprocess.Popen.
    Given a string of a bash command, run the shell command here.
    This is risky AF, so make sure you are careful and don't use this to accept user input.
    """
    args = shlex.split(command)
    logger.debug(f"args: {args}")
    env = {
        **os.environ,
        # "TEST_VARIABLE": str(1234),
    }
    if env_vars:
        env.update(env_vars)
    # https://docs.python.org/3/library/subprocess.html#popen-constructor
    if wait_for_finish:
        process = subprocess.run(
            args=args,
            shell=shell,  # nosec - skip B404:blacklist, B603:subprocess_without_shell_equals_true
            timeout=timeout,
            stdout=subprocess.PIPE,
            stdin=subprocess.PIPE,
            text=True,
            universal_newlines=True,
            env=env
        )
    else:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore", category=ResourceWarning)
            process = subprocess.Popen(
                args=args,
                shell=shell,  # nosec - skip B404:blacklist, B603:subprocess_without_shell_equals_true
                stdout=subprocess.PIPE,
                stdin=subprocess.PIPE,
                text=True,
                encoding="ascii",
                universal_newlines=True,
                env=env
            )
    logger.debug(f"{command} response: {process.stdout}")
    return process


def run_click_command(click_function_to_run, args: str, env: dict = None) -> Result:
    """
    Given a python method that runs a click command, use the click.testing.CliRunner to run the command with arguments provided as a string
    """
    runner = CliRunner()
    logger.debug(f"args: '{args}")
    args = shlex.split(args)
    response = runner.invoke(cli=click_function_to_run, args=args, catch_exceptions=False, env=env)
    return response



import unittest
import logging
from kinnaird_utils.time import get_year_month_day_string

logger = logging.getLogger(__name__)


class TimeTestCase(unittest.TestCase):
    def test_get_year_month_day_string(self):
        """this is pretty much just to see the output while I am writing this, not to verify results or be a solid check at all"""
        results = get_year_month_day_string()
        print(results)




import re


def strip_special_characters(some_string: str) -> str:
    """Remove all special characters, punctuation, and spaces from a string"""
    # Input: "Special $#! characters   spaces 888323"
    # Output: 'Specialcharactersspaces888323'
    result = ''.join(e for e in some_string if e.isalnum())
    return result


def chomp(string: str) -> str:
    """This chomp cleans up all white-space, not just at the ends"""
    string = str(string)
    result = string.replace("\n", " ")  # Convert line ends to spaces
    result = re.sub(" [ ]*", " ", result)  # Truncate multiple spaces to single space
    result = result.replace(" ", "")
    result = result.replace(u"\xa0", u" ")  # Remove non-breaking space
    result = re.sub("^[ ]*", "", result)  # Clean start
    return re.sub("[ ]*$", "", result)  # Clean end


def chomp_keep_single_spaces(string: str) -> str:
    """This chomp cleans up all white-space, not just at the ends"""
    string = str(string)
    result = string.replace("\n", " ")  # Convert line ends to spaces
    result = re.sub(" [ ]*", " ", result)  # Truncate multiple spaces to single space
    result = result.replace(" ", " ")  # Replace weird spaces with regular spaces
    result = result.replace(u"\xa0", u" ")  # Remove non-breaking space
    result = re.sub("^[ ]*", "", result)  # Clean start
    return re.sub("[ ]*$", "", result)  # Clean end


def parse_text_for_url(string: str) -> str:
    return re.search("(?P<url>https?://[^\s'\"]+)", string).group("url")



import logging
import boto3
logger = logging.getLogger(__name__)


def get_available_regions_for_service(service: str):
    """AWS exposes their list of regions as an API. Gather the list."""
    regions = boto3.session.Session().get_available_regions(service)
    logger.debug("The service %s does not have available regions. Returning us-east-1 as default")
    if not regions:
        regions = ["us-east-1"]
    return regions



import os


def get_filenames_in_folder(path: str) -> list:
    """Given a folder path, get a list of filenames in that folder."""
    file_list = []
    for filename in os.listdir(path):
        if os.path.isfile(os.path.join(path, filename)):
            if filename not in file_list:
                file_list.append(filename)
    file_list.sort()
    return file_list


def get_subfolder_names_in_folder(path: str) -> list:
    """Get a list of subfolder names inside a folder."""
    folder_list = []
    for name in os.listdir(path):
        if os.path.isdir(os.path.join(path, name)):
            folder_list.append(name)
    folder_list.sort()
    return folder_list



import os
import unittest
import json
import logging
from kinnaird_utils.files import read_yaml_file

logger = logging.getLogger(__name__)


class FileTestCase(unittest.TestCase):
    def setUp(self) -> None:
        self.example_file = os.path.abspath(os.path.join(
            os.path.dirname(__file__),
            "files",
            "example.yml"
        ))

    def test_read_yaml_file(self):
        results = read_yaml_file(self.example_file)
        print(json.dumps(results, indent=4))
        expected_results = {
            "first_name": "Kinnaird",
            "last_name": "McQuade"
        }
        self.assertDictEqual(results, expected_results)


import unittest
import logging
from kinnaird_utils import text_parser_utils

logger = logging.getLogger(__name__)


def get_input(text):
    return input(text)


def answer():
    ans = get_input('enter yes or no')
    if ans == 'yes':
        return 'you entered yes'
    if ans == 'no':
        return 'you entered no'


class TextTestCase(unittest.TestCase):
    def test_strip_special_characters(self):
        input_string = "Special $#! characters   spaces 888323"
        result = text_parser_utils.strip_special_characters(input_string)
        expected_output = "Specialcharactersspaces888323"
        self.assertEqual(result, expected_output)

    def test_parse_text_for_url(self):
        # Case: Base URL
        some_args = f"-u 'http://testphp.vulnweb.com' --crawl -l 3 --params --blind --skip"
        result = text_parser_utils.parse_text_for_url(string=some_args)
        self.assertEqual(result, "http://testphp.vulnweb.com")
        # Case: /sup.html
        some_args = f"-u 'http://testphp.vulnweb.com/sup.html' --crawl -l 3 --params --blind --skip"
        result = text_parser_utils.parse_text_for_url(string=some_args)
        self.assertEqual(result, "http://testphp.vulnweb.com/sup.html")
        # Case: With query parameters
        some_args = f"-u 'http://testphp.vulnweb.com/sup.html?bruh' --crawl -l 3 --params --blind --skip"
        result = text_parser_utils.parse_text_for_url(string=some_args)
        self.assertEqual(result, "http://testphp.vulnweb.com/sup.html?bruh")




import os
import unittest
import json
import logging
import subprocess
import warnings
from policy_sentry.command.query import query
from policy_sentry.command.write_policy import write_policy
from kinnaird_utils.shell import run_click_command, run_shell_command
from kinnaird_utils.testing_utils import ignore_warnings
from kinnaird_utils.log_formatting import set_stream_logger
logger = logging.getLogger(__name__)
warnings.simplefilter("ignore", ResourceWarning)


class ShellTestCase(unittest.TestCase):

    def test_run_shell_command_wait(self):
        command = "echo 'sup'"
        results = run_shell_command(command=command, timeout=5, shell=False)
        print(results.__dict__)
        self.assertTrue(results.stdout == "sup\n")
        self.assertTrue(results.returncode == 0)

    @ignore_warnings
    def test_run_shell_command_dont_wait(self):
        command = "echo 'sup'"
        # unittest.main(warnings='ignore')
        results = run_shell_command(command=command, timeout=5, shell=False, wait_for_finish=False)
        print(results.__dict__)
        print(results.stdout.__dict__)

    def test_run_click_command(self):
        results = run_click_command(query, args="action-table --service ram")
        print(results.output)
        # Expected results:
        """
ALL ram actions:
ram:AcceptResourceShareInvitation
ram:AssociateResourceShare
ram:AssociateResourceSharePermission
ram:CreateResourceShare
ram:DeleteResourceShare
ram:DisassociateResourceShare
ram:DisassociateResourceSharePermission
ram:EnableSharingWithAwsOrganization
ram:GetPermission
ram:GetResourcePolicies
ram:GetResourceShareAssociations
ram:GetResourceShareInvitations
ram:GetResourceShares
ram:ListPendingInvitationResources
ram:ListPermissions
ram:ListPrincipals
ram:ListResourceSharePermissions
ram:ListResourceTypes
ram:ListResources
ram:PromoteResourceShareCreatedFromPolicy
ram:RejectResourceShareInvitation
ram:TagResource
ram:UntagResource
ram:UpdateResourceShare

        """
        self.assertTrue("ram:GetPermission" in results.output)

    def test_run_click_command_fail(self):
        with self.assertRaises(FileNotFoundError):
            results = run_click_command(write_policy, args="--input-file doesnotexist.yml")



import os
import logging
import csv
import json
import yaml

logger = logging.getLogger(__name__)


def read_yaml_file(filename: str) -> dict:
    """
    Reads a YAML file, safe loads, and returns the dictionary

    :param filename: name of the yaml file
    :return: dictionary of YAML file contents
    """
    with open(filename, "r") as yaml_file:
        cfg = yaml.safe_load(yaml_file)
    return cfg


def read_csv_file(filename: str, delimiter: str = ",") -> list:
    results = []
    with open(filename) as csv_file:
        csv_reader = csv.DictReader(csv_file, delimiter=delimiter)
        for row in csv_reader:
            results.append(row)
    return results


def read_json_file(file: str) -> dict:
    with open(file) as f:
        contents = f.read()
        try:
            results = json.loads(contents)
        except json.decoder.JSONDecodeError as error:
            logger.debug(error)
            decoded_data = contents.encode().decode("utf-8-sig")
            results = json.loads(decoded_data)
    return results


def write_json_to_file(file: str, content: str) -> None:
    if os.path.exists(file):
        logger.debug("%s exists. Removing the file and replacing its contents." % file)
        os.remove(file)

    with open(file, "w") as f:
        json.dump(content, f, indent=4)


def write_file(file: str, content: str) -> None:
    if os.path.exists(file):
        logger.debug("%s exists. Removing the file and replacing its contents." % file)
        os.remove(file)
    with open(file, "w") as f:
        f.write(content)


def read_file(file: str) -> str:
    with open(file, "r") as f:
        content = f.read()
    return content


def remove_file_if_exists(file: str) -> None:
    if os.path.exists(file):
        logger.debug("%s exists. Removing" % file)
        os.remove(file)



import logging
logger = logging.getLogger(__name__)
logger.setLevel(logging.DEBUG)

VERBOSE_LOG_FORMAT = "%(asctime)s %(name)s [%(levelname)s] %(message)s"
SIMPLE_LOG_FORMAT = "[%(levelname)s] %(message)s"


def set_stream_logger(name="kinnaird_utils", level=logging.DEBUG, format_string=None):  # pylint: disable=redefined-outer-name
    """
    Add a stream handler for the given name and level to the logging module.
    By default, this logs all cloudsplaining messages to ``stdout``.
        >>> import kinnaird_utils
        >>> kinnaird_utils.log_formatting.set_stream_logger('cloudsplaining.scan', logging.INFO)
    :type name: string
    :param name: Log name
    :type level: int
    :param level: Logging level, e.g. ``logging.INFO``
    :type format_string: str
    :param format_string: Log message format
    """
    # remove existing handlers. since NullHandler is added by default
    handlers = logging.getLogger(name).handlers
    for handler in handlers:  # pylint: disable=redefined-outer-name
        logging.getLogger(name).removeHandler(handler)
    if format_string is None:
        format_string = "%(asctime)s %(name)s [%(levelname)s] %(message)s"
    logger = logging.getLogger(name)  # pylint: disable=redefined-outer-name
    logger.setLevel(level)
    handler = logging.StreamHandler()  # pylint: disable=redefined-outer-name
    handler.setLevel(level)
    formatter = logging.Formatter(format_string)  # pylint: disable=redefined-outer-name
    handler.setFormatter(formatter)
    logger.addHandler(handler)


def set_log_level(verbose: int):
    """
    Set Log Level based on click's count argument.

    Default log level to critical; otherwise, set to: warning for -v, info for -vv, debug for -vvv

    :param verbose: integer for verbosity count.
    :return:
    """
    if verbose == 1:
        set_stream_logger(level=getattr(logging, "WARNING"))
    elif verbose == 2:
        set_stream_logger(level=getattr(logging, "INFO"))
    elif verbose >= 3:
        set_stream_logger(level=getattr(logging, "DEBUG"))
    else:
        set_stream_logger(level=getattr(logging, "CRITICAL"))


def set_log_format_to_simple_warning(name: str, level: int = logging.WARNING):
    """A basic wrapper around set_stream_logger that sets the log formatter to a simple formatted warning"""
    format_string = "[%(levelname)s] %(message)s"
    set_stream_logger(name=name, level=level, format_string=format_string)


def remove_log_messages_from_cli_output(cli_output_string: str) -> str:
    """Given a string of CLI output, remove the log messages"""
    results = []
    for line in cli_output_string.splitlines():
        if (
                not line.startswith("[DEBUG]")
                and not line.startswith("[INFO]")
                and not line.startswith("[WARN]")
                and not line.startswith("[WARNING]")
                and not line.startswith("[ERROR]")
                and not line.startswith("[CRITICAL]")
        ):
            results.append(line)
    results = "\n".join(results)
    logger.info(results)
    return results



# # https://psutil.readthedocs.io/en/latest/#recipes
# import os
# import psutil
# import signal
#
#
# def find_processes_by_name(name: str) -> list:
#     """Return a list of processes matching 'name'."""
#     ls = []
#     for p in psutil.process_iter(["name", "exe", "cmdline"]):
#         if name == p.info['name'] or \
#                 p.info['exe'] and os.path.basename(p.info['exe']) == name or \
#                 p.info['cmdline'] and p.info['cmdline'][0] == name:
#             ls.append(p)
#     return ls
#
#
# # def kill_process_id(proc_pid: int, recursive: bool = True):
# def kill_process_id(proc_pid: int):
#     """Kill a process ID"""
#     process = psutil.Process(proc_pid)
#     # for proc in process.children(recursive=recursive):
#     for proc in process.children(recursive=True):
#         proc.kill()
#     process.kill()
#
#
# def kill_process_tree(pid: int, sig=signal.SIGTERM, include_parent=True, timeout=None, on_terminate=None) -> tuple:
#     """
#     Kill a process tree (including grandchildren) with signal "sig" and return a (gone, still_alive) tuple.
#
#     "on_terminate", if specified, is a callback function which is called as soon as a child terminates.
#     """
#     assert pid != os.getpid(), "won't kill myself"  # nosec - skip B101:assert_used
#     parent = psutil.Process(pid)
#     children = parent.children(recursive=True)
#     if include_parent:
#         children.append(parent)
#     for p in children:
#         try:
#             p.send_signal(sig)
#         except psutil.NoSuchProcess:
#             pass
#     gone, alive = psutil.wait_procs(children, timeout=timeout,
#                                     callback=on_terminate)
#     return (gone, alive)



import logging
import boto3
logger = logging.getLogger(__name__)


def get_object_from_s3(bucket_name: str, object_key: str, s3_client: boto3.Session.client):
    logger.info(f"Downloading object: s3://{bucket_name}/{object_key}")
    response = s3_client.get_object(Bucket=bucket_name, Key=object_key)
    content = response['Body'].read().decode('utf-8')
    return content


def write_simple_object_to_s3(content: str, bucket_name: str, object_key: str, s3_client: boto3.Session.client):
    """Write an object to S3 with basic AES256 encryption"""
    logger.info(f"Writing object: s3://{bucket_name}/{object_key}")
    response = s3_client.put_object(
        ACL="private",
        Bucket=bucket_name,
        Key=object_key,
        ServerSideEncryption="AES256",
        Body=content
    )
    logger.debug(response)



import os


def is_inside_docker():
    path = '/proc/self/cgroup'
    return (
        os.path.exists('/.dockerenv') or
        os.path.isfile(path) and any('docker' in line for line in open(path))
    )



import warnings


def ignore_warnings(test_func):
    """
    Use a decorator to skip warnings in your unit tests.

    See the solution here for more details: https://stackoverflow.com/questions/26563711/disabling-python-3-2-resourcewarning#answer-26620811
    :param test_func:
    :return:
    """
    def do_test(self, *args, **kwargs):
        with warnings.catch_warnings():
            warning_type = kwargs.get("warning_type", ResourceWarning)
            warnings.simplefilter("ignore", warning_type)
            test_func(self, *args, **kwargs)
    return do_test



"""Setup script"""
import setuptools
import os
import re

HERE = os.path.abspath(os.path.dirname(__file__))
VERSION_RE = re.compile(r"""__version__ = ['"]([0-9.]+)['"]""")
TESTS_REQUIRE = ["coverage", "nose", "pytest"]
DESCRIPTION = ""
REQUIRED_PACKAGES = [
    "pyyaml",
    "click",
    "pydantic",
    "boto3",
    "botocore",
    "aws-lambda-powertools",
    # "psutil",
    "colorama"
]
PROJECT_URLS = {
    "Documentation": "https://policy-sentry.readthedocs.io/",
    "Code": "https://github.com/salesforce/policy_sentry/",
    "Example Report": "https://opensource.salesforce.com/cloudsplaining",
    "Twitter": "https://twitter.com/kmcquade3",
    # "Example Report": "https://opensource.salesforce.com/policy_sentry",
    "Red Team Report": "https://kmcquade.com/security.html"
}


def get_version():
    init = open(os.path.join(HERE, "kinnaird_utils", "version.py")).read()
    return VERSION_RE.search(init).group(1)


def get_description():
    return open(
        os.path.join(os.path.abspath(HERE), "README.md"), encoding="utf-8"
    ).read()


setuptools.setup(
    name="kinnaird-utils",
    include_package_data=True,
    version=get_version(),
    author="Kinnaird McQuade",
    author_email="kinnairdm@gmail.com",
    description=DESCRIPTION,
    long_description=get_description(),
    long_description_content_type="text/markdown",
    url="https://github.com/kmcquade/kinnaird-utils",
    packages=setuptools.find_packages(exclude=["test*"]),
    tests_require=TESTS_REQUIRE,
    install_requires=REQUIRED_PACKAGES,
    project_urls=PROJECT_URLS,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    # entry_points={"console_scripts": "azure-guardrails=azure_guardrails.bin.cli:main"},
    zip_safe=True,
    keywords="kinnaird",
    python_requires=">=3.7",
)



from kinnaird_utils.aws.login import get_boto3_client


def get_latest_ecr_image_tag(repository_name: str, region: str, profile: str = None) -> str:
    """aws ecr describe-images --profile personal --region us-east-1 --output json --repository-name zap --query 'sort_by(imageDetails,& imagePushedAt)[-1].imageTags[0]'"""
    client = get_boto3_client(service="ecr", region=region, profile=profile)
    response = client.describe_images(repositoryName=repository_name, filter={"tagStatus": "TAGGED"})
    adjusted = sorted(response["imageDetails"], key=lambda i: int(i['imagePushedAt']))
    latest_image_tag = adjusted[-1]["imageTags"][0]
    return latest_image_tag



from datetime import date


def get_year_month_day_string() -> str:
    today = date.today()
    d1 = today.strftime("%Y-%m-%d")
    return d1

    # # dd/mm/YY, like 16/09/2019
    # d1 = today.strftime("%d/%m/%Y")
    # print("d1 =", d1)
    #
    # # Textual month, day and year, like September 16, 2019
    # d2 = today.strftime("%B %d, %Y")
    # print("d2 =", d2)
    #
    # # mm/dd/y, like 09/16/19
    # d3 = today.strftime("%m/%d/%y")
    # print("d3 =", d3)
    #
    # # Month abbreviation, day and year, like Sep-16-2019
    # d4 = today.strftime("%b-%d-%Y")
    # print("d4 =", d4)